# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2018


import os
import sys

import opentracing as ot
import opentracing.ext.tags as ext
import wrapt

from ...log import logger
from ...singletons import agent, tracer
from ...util.secrets import strip_secrets_from_query

DJ_INSTANA_MIDDLEWARE = 'instana.instrumentation.django.middleware.InstanaMiddleware'

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object


class InstanaMiddleware(MiddlewareMixin):
    """ Django Middleware to provide request tracing for Instana """

    def __init__(self, get_response=None):
        super(InstanaMiddleware, self).__init__(get_response)
        self.get_response = get_response

    def _extract_custom_headers(self, span, headers, format):
        if agent.options.extra_http_headers is None:
            return

        try:            
            for custom_header in agent.options.extra_http_headers:
                # Headers are available in this format: HTTP_X_CAPTURE_THIS
                django_header = ('HTTP_' + custom_header.upper()).replace('-', '_') if format else custom_header

                if django_header in headers:
                    span.set_tag("http.header.%s" % custom_header, headers[django_header])

        except Exception:
            logger.debug("extract_custom_headers: ", exc_info=True)

    def process_request(self, request):
        try:
            env = request.environ

            ctx = tracer.extract(ot.Format.HTTP_HEADERS, env)
            request.iscope = tracer.start_active_span('django', child_of=ctx)

            self._extract_custom_headers(request.iscope.span, env, format=True)

            request.iscope.span.set_tag(ext.HTTP_METHOD, request.method)
            if 'PATH_INFO' in env:
                request.iscope.span.set_tag(ext.HTTP_URL, env['PATH_INFO'])
            if 'QUERY_STRING' in env and len(env['QUERY_STRING']):
                scrubbed_params = strip_secrets_from_query(env['QUERY_STRING'], agent.options.secrets_matcher,
                                                           agent.options.secrets_list)
                request.iscope.span.set_tag("http.params", scrubbed_params)
            if 'HTTP_HOST' in env:
                request.iscope.span.set_tag("http.host", env['HTTP_HOST'])
        except Exception:
            logger.debug("Django middleware @ process_request", exc_info=True)

    def process_response(self, request, response):
        try:
            if request.iscope is not None:
                if 500 <= response.status_code:
                    request.iscope.span.assure_errored()
                # for django >= 2.2
                if request.resolver_match is not None and hasattr(request.resolver_match, 'route'):
                    path_tpl = request.resolver_match.route
                # django < 2.2 or in case of 404
                else:
                    try:
                        from django.urls import resolve
                        view_name = resolve(request.path)._func_path
                        path_tpl = "".join(self.__url_pattern_route(view_name))
                    except Exception:
                        # the resolve method can fire a Resolver404 exception, in this case there is no matching route
                        # so the path_tpl is set to None in order not to be added as a tag
                        path_tpl = None
                if path_tpl:
                    request.iscope.span.set_tag("http.path_tpl", path_tpl)

                request.iscope.span.set_tag(ext.HTTP_STATUS_CODE, response.status_code)
                self._extract_custom_headers(request.iscope.span, response.headers, format=False)
                tracer.inject(request.iscope.span.context, ot.Format.HTTP_HEADERS, response)
                response['Server-Timing'] = "intid;desc=%s" % request.iscope.span.context.trace_id
        except Exception:
            logger.debug("Instana middleware @ process_response", exc_info=True)
        finally:
            if request.iscope is not None:
                request.iscope.close()
                request.iscope = None
        return response

    def process_exception(self, request, exception):
        from django.http.response import Http404

        if isinstance(exception, Http404):
            return None

        if request.iscope is not None:
            request.iscope.span.log_exception(exception)

    def __url_pattern_route(self, view_name):
        from django.conf import settings
        from django.urls import RegexURLResolver as URLResolver

        urlconf = __import__(settings.ROOT_URLCONF, {}, {}, [''])

        def list_urls(urlpatterns, parent_pattern=None):
            if not urlpatterns:
                return
            if parent_pattern is None:
                parent_pattern = []
            first = urlpatterns[0]
            if isinstance(first, URLPattern):
                if first.lookup_str == view_name:
                    if hasattr(first, "regex"):
                        return parent_pattern + [str(first.regex.pattern)]
                    else:
                        return parent_pattern + [str(first.pattern)]
            elif isinstance(first, URLResolver):
                if hasattr(first, "regex"):
                    return list_urls(first.url_patterns, parent_pattern + [str(first.regex.pattern)])
                else:
                    return list_urls(first.url_patterns, parent_pattern + [str(first.pattern)])
            return list_urls(urlpatterns[1:], parent_pattern)

        return list_urls(urlconf.urlpatterns)


def load_middleware_wrapper(wrapped, instance, args, kwargs):
    try:
        from django.conf import settings

        # Django >=1.10 to <2.0 support old-style MIDDLEWARE_CLASSES so we
        # do as well here
        if hasattr(settings, 'MIDDLEWARE') and settings.MIDDLEWARE is not None:
            if DJ_INSTANA_MIDDLEWARE in settings.MIDDLEWARE:
                return wrapped(*args, **kwargs)

            if isinstance(settings.MIDDLEWARE, tuple):
                settings.MIDDLEWARE = (DJ_INSTANA_MIDDLEWARE,) + settings.MIDDLEWARE
            elif isinstance(settings.MIDDLEWARE, list):
                settings.MIDDLEWARE = [DJ_INSTANA_MIDDLEWARE] + settings.MIDDLEWARE
            else:
                logger.warning("Instana: Couldn't add InstanaMiddleware to Django")

        elif hasattr(settings, 'MIDDLEWARE_CLASSES') and settings.MIDDLEWARE_CLASSES is not None:
            if DJ_INSTANA_MIDDLEWARE in settings.MIDDLEWARE_CLASSES:
                return wrapped(*args, **kwargs)

            if isinstance(settings.MIDDLEWARE_CLASSES, tuple):
                settings.MIDDLEWARE_CLASSES = (DJ_INSTANA_MIDDLEWARE,) + settings.MIDDLEWARE_CLASSES
            elif isinstance(settings.MIDDLEWARE_CLASSES, list):
                settings.MIDDLEWARE_CLASSES = [DJ_INSTANA_MIDDLEWARE] + settings.MIDDLEWARE_CLASSES
            else:
                logger.warning("Instana: Couldn't add InstanaMiddleware to Django")

        else:
            logger.warning("Instana: Couldn't find middleware settings")

        return wrapped(*args, **kwargs)
    except Exception:
        logger.warning("Instana: Couldn't add InstanaMiddleware to Django: ", exc_info=True)


try:
    if 'django' in sys.modules:
        logger.debug("Instrumenting django")
        wrapt.wrap_function_wrapper('django.core.handlers.base', 'BaseHandler.load_middleware', load_middleware_wrapper)

        if '/tmp/.instana/python' in sys.path:
            # If we are instrumenting via AutoTrace (in an already running process), then the
            # WSGI middleware has to be live reloaded.
            from django.core.servers.basehttp import get_internal_wsgi_application
            from django.core.exceptions import ImproperlyConfigured

            try:
                wsgiapp = get_internal_wsgi_application()
                wsgiapp.load_middleware()
            except ImproperlyConfigured:
                pass

except Exception:
    logger.debug("django.middleware:", exc_info=True)
    pass



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import os
import unittest
import sys

from mock import patch

from instana.tracer import InstanaTracer
from instana.recorder import StanRecorder
from instana.agent.host import HostAgent
from instana.collector.helpers.runtime import PATH_OF_AUTOTRACE_WEBHOOK_SITEDIR
from instana.collector.host import HostCollector
from instana.singletons import get_agent, set_agent, get_tracer, set_tracer
from instana.version import VERSION

class TestHostCollector(unittest.TestCase):
    def __init__(self, methodName='runTest'):
        super(TestHostCollector, self).__init__(methodName)
        self.agent = None
        self.span_recorder = None
        self.tracer = None

        self.original_agent = get_agent()
        self.original_tracer = get_tracer()

    def setUp(self):
        self.webhook_sitedir_path = PATH_OF_AUTOTRACE_WEBHOOK_SITEDIR + '3.8.0'

    def tearDown(self):
        """ Reset all environment variables of consequence """
        variable_names = (
                "AWS_EXECUTION_ENV", "INSTANA_EXTRA_HTTP_HEADERS",
                "INSTANA_ENDPOINT_URL", "INSTANA_AGENT_KEY", "INSTANA_ZONE",
                "INSTANA_TAGS", "INSTANA_DISABLE_METRICS_COLLECTION",
                "INSTANA_DISABLE_PYTHON_PACKAGE_COLLECTION",
                "AUTOWRAPT_BOOTSTRAP"
                )

        for variable_name in variable_names:
            if variable_name in os.environ:
                os.environ.pop(variable_name)

        set_agent(self.original_agent)
        set_tracer(self.original_tracer)
        if self.webhook_sitedir_path in sys.path:
            sys.path.remove(self.webhook_sitedir_path)

    def create_agent_and_setup_tracer(self):
        self.agent = HostAgent()
        self.span_recorder = StanRecorder(self.agent)
        self.tracer = InstanaTracer(recorder=self.span_recorder)
        set_agent(self.agent)
        set_tracer(self.tracer)

    def test_prepare_payload_basics(self):
        self.create_agent_and_setup_tracer()

        payload = self.agent.collector.prepare_payload()
        self.assertTrue(payload)

        self.assertEqual(len(payload.keys()), 3)
        self.assertIn('spans', payload)
        self.assertIsInstance(payload['spans'], list)
        self.assertEqual(len(payload['spans']), 0)
        self.assertIn('metrics', payload)
        self.assertEqual(len(payload['metrics'].keys()), 1)
        self.assertIn('plugins', payload['metrics'])
        self.assertIsInstance(payload['metrics']['plugins'], list)
        self.assertEqual(len(payload['metrics']['plugins']), 1)

        python_plugin = payload['metrics']['plugins'][0]
        self.assertEqual(python_plugin['name'], 'com.instana.plugin.python')
        self.assertEqual(python_plugin['entityId'], str(os.getpid()))
        self.assertIn('data', python_plugin)
        self.assertIn('snapshot', python_plugin['data'])
        self.assertIn('m', python_plugin['data']['snapshot'])
        self.assertEqual('Manual', python_plugin['data']['snapshot']['m'])
        self.assertIn('metrics', python_plugin['data'])

        # Validate that all metrics are reported on the first run
        self.assertIn('ru_utime', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_utime']), [float, int])
        self.assertIn('ru_stime', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_stime']), [float, int])
        self.assertIn('ru_maxrss', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_maxrss']), [float, int])
        self.assertIn('ru_ixrss', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_ixrss']), [float, int])
        self.assertIn('ru_idrss', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_idrss']), [float, int])
        self.assertIn('ru_isrss', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_isrss']), [float, int])
        self.assertIn('ru_minflt', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_minflt']), [float, int])
        self.assertIn('ru_majflt', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_majflt']), [float, int])
        self.assertIn('ru_nswap', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_nswap']), [float, int])
        self.assertIn('ru_inblock', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_inblock']), [float, int])
        self.assertIn('ru_oublock', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_oublock']), [float, int])
        self.assertIn('ru_msgsnd', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_msgsnd']), [float, int])
        self.assertIn('ru_msgrcv', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_msgrcv']), [float, int])
        self.assertIn('ru_nsignals', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_nsignals']), [float, int])
        self.assertIn('ru_nvcsw', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_nvcsw']), [float, int])
        self.assertIn('ru_nivcsw', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['ru_nivcsw']), [float, int])
        self.assertIn('alive_threads', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['alive_threads']), [float, int])
        self.assertIn('dummy_threads', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['dummy_threads']), [float, int])
        self.assertIn('daemon_threads', python_plugin['data']['metrics'])
        self.assertIn(type(python_plugin['data']['metrics']['daemon_threads']), [float, int])

        self.assertIn('gc', python_plugin['data']['metrics'])
        self.assertIsInstance(python_plugin['data']['metrics']['gc'], dict)
        self.assertIn('collect0', python_plugin['data']['metrics']['gc'])
        self.assertIn(type(python_plugin['data']['metrics']['gc']['collect0']), [float, int])
        self.assertIn('collect1', python_plugin['data']['metrics']['gc'])
        self.assertIn(type(python_plugin['data']['metrics']['gc']['collect1']), [float, int])
        self.assertIn('collect2', python_plugin['data']['metrics']['gc'])
        self.assertIn(type(python_plugin['data']['metrics']['gc']['collect2']), [float, int])
        self.assertIn('threshold0', python_plugin['data']['metrics']['gc'])
        self.assertIn(type(python_plugin['data']['metrics']['gc']['threshold0']), [float, int])
        self.assertIn('threshold1', python_plugin['data']['metrics']['gc'])
        self.assertIn(type(python_plugin['data']['metrics']['gc']['threshold1']), [float, int])
        self.assertIn('threshold2', python_plugin['data']['metrics']['gc'])
        self.assertIn(type(python_plugin['data']['metrics']['gc']['threshold2']), [float, int])

    def test_prepare_payload_basics_disable_runtime_metrics(self):
        os.environ["INSTANA_DISABLE_METRICS_COLLECTION"] = "TRUE"
        self.create_agent_and_setup_tracer()

        payload = self.agent.collector.prepare_payload()
        self.assertTrue(payload)

        self.assertEqual(len(payload.keys()), 3)
        self.assertIn('spans', payload)
        self.assertIsInstance(payload['spans'], list)
        self.assertEqual(len(payload['spans']), 0)
        self.assertIn('metrics', payload)
        self.assertEqual(len(payload['metrics'].keys()), 1)
        self.assertIn('plugins', payload['metrics'])
        self.assertIsInstance(payload['metrics']['plugins'], list)
        self.assertEqual(len(payload['metrics']['plugins']), 1)

        python_plugin = payload['metrics']['plugins'][0]
        self.assertEqual(python_plugin['name'], 'com.instana.plugin.python')
        self.assertEqual(python_plugin['entityId'], str(os.getpid()))
        self.assertIn('data', python_plugin)
        self.assertIn('snapshot', python_plugin['data'])
        self.assertIn('m', python_plugin['data']['snapshot'])
        self.assertEqual('Manual', python_plugin['data']['snapshot']['m'])
        self.assertNotIn('metrics', python_plugin['data'])

    @patch.object(HostCollector, "should_send_snapshot_data")
    def test_prepare_payload_with_snapshot_with_python_packages(self, mock_should_send_snapshot_data):
        mock_should_send_snapshot_data.return_value = True
        self.create_agent_and_setup_tracer()

        payload = self.agent.collector.prepare_payload()
        self.assertTrue(payload)
        self.assertIn('snapshot', payload['metrics']['plugins'][0]['data'])
        snapshot =  payload['metrics']['plugins'][0]['data']['snapshot']
        self.assertTrue(snapshot)
        self.assertIn('m', snapshot)
        self.assertEqual('Manual', snapshot['m'])
        self.assertIn('version', snapshot)
        self.assertGreater(len(snapshot['versions']), 5)
        self.assertEqual(snapshot['versions']['instana'], VERSION)
        self.assertIn('wrapt', snapshot['versions'])
        self.assertIn('fysom', snapshot['versions'])
        self.assertIn('opentracing', snapshot['versions'])
        self.assertIn('basictracer', snapshot['versions'])

    @patch.object(HostCollector, "should_send_snapshot_data")
    def test_prepare_payload_with_snapshot_disabled_python_packages(self, mock_should_send_snapshot_data):
        mock_should_send_snapshot_data.return_value = True
        os.environ["INSTANA_DISABLE_PYTHON_PACKAGE_COLLECTION"] = "TRUE"
        self.create_agent_and_setup_tracer()

        payload = self.agent.collector.prepare_payload()
        self.assertTrue(payload)
        self.assertIn('snapshot', payload['metrics']['plugins'][0]['data'])
        snapshot =  payload['metrics']['plugins'][0]['data']['snapshot']
        self.assertTrue(snapshot)
        self.assertIn('m', snapshot)
        self.assertEqual('Manual', snapshot['m'])
        self.assertIn('version', snapshot)
        self.assertEqual(len(snapshot['versions']), 1)
        self.assertEqual(snapshot['versions']['instana'], VERSION)


    @patch.object(HostCollector, "should_send_snapshot_data")
    def test_prepare_payload_with_autowrapt(self, mock_should_send_snapshot_data):
        mock_should_send_snapshot_data.return_value = True
        os.environ["AUTOWRAPT_BOOTSTRAP"] = "instana"
        self.create_agent_and_setup_tracer()

        payload = self.agent.collector.prepare_payload()
        self.assertTrue(payload)
        self.assertIn('snapshot', payload['metrics']['plugins'][0]['data'])
        snapshot =  payload['metrics']['plugins'][0]['data']['snapshot']
        self.assertTrue(snapshot)
        self.assertIn('m', snapshot)
        self.assertEqual('Autowrapt', snapshot['m'])
        self.assertIn('version', snapshot)
        self.assertGreater(len(snapshot['versions']), 5)
        expected_packages = ('instana', 'wrapt', 'fysom', 'opentracing', 'basictracer')
        for package in expected_packages:
            self.assertIn(package, snapshot['versions'], f"{package} not found in snapshot['versions']")
        self.assertEqual(snapshot['versions']['instana'], VERSION)


    @patch.object(HostCollector, "should_send_snapshot_data")
    def test_prepare_payload_with_autotrace(self, mock_should_send_snapshot_data):
        mock_should_send_snapshot_data.return_value = True

        sys.path.append(self.webhook_sitedir_path)

        self.create_agent_and_setup_tracer()

        payload = self.agent.collector.prepare_payload()
        self.assertTrue(payload)
        self.assertIn('snapshot', payload['metrics']['plugins'][0]['data'])
        snapshot =  payload['metrics']['plugins'][0]['data']['snapshot']
        self.assertTrue(snapshot)
        self.assertIn('m', snapshot)
        self.assertEqual('AutoTrace', snapshot['m'])
        self.assertIn('version', snapshot)
        self.assertGreater(len(snapshot['versions']), 5)
        expected_packages = ('instana', 'wrapt', 'fysom', 'opentracing', 'basictracer')
        for package in expected_packages:
            self.assertIn(package, snapshot['versions'], f"{package} not found in snapshot['versions']")
        self.assertEqual(snapshot['versions']['instana'], VERSION)



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import unittest
import signal
import os

from instana.autoprofile.profiler import Profiler
from instana.autoprofile.runtime import runtime_info, register_signal


class RuntimeTestCase(unittest.TestCase):

    def test_register_signal(self):
        if runtime_info.OS_WIN:
            return

        result = {'handler': 0}

        def _handler(signum, frame):
            result['handler'] += 1

        register_signal(signal.SIGUSR1, _handler)

        os.kill(os.getpid(), signal.SIGUSR1)
        os.kill(os.getpid(), signal.SIGUSR1)

        signal.signal(signal.SIGUSR1, signal.SIG_DFL)

        self.assertEqual(result['handler'], 2)


    '''def test_register_signal_default(self):
        result = {'handler': 0}

        def _handler(signum, frame):
            result['handler'] += 1

        register_signal(signal.SIGUSR1, _handler, once = True)

        os.kill(os.getpid(), signal.SIGUSR1)
        os.kill(os.getpid(), signal.SIGUSR1)

        self.assertEqual(result['handler'], 1)'''


if __name__ == '__main__':
    unittest.main()




# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import unittest

from ..helpers import testenv
from instana.singletons import agent, tracer

from sqlalchemy.orm import sessionmaker
from sqlalchemy.exc import OperationalError
from sqlalchemy.orm import declarative_base
from sqlalchemy import Column, Integer, String, create_engine, text


engine = create_engine("postgresql://%s:%s@%s/%s" % (testenv['postgresql_user'], testenv['postgresql_pw'],
                                                     testenv['postgresql_host'], testenv['postgresql_db']))
Base = declarative_base()

class StanUser(Base):
     __tablename__ = 'churchofstan'

     id = Column(Integer, primary_key=True)
     name = Column(String)
     fullname = Column(String)
     password = Column(String)

     def __repr__(self):
        return "<User(name='%s', fullname='%s', password='%s')>" % (
                             self.name, self.fullname, self.password)

Base.metadata.create_all(engine)

stan_user = StanUser(name='IAmStan', fullname='Stan Robot', password='3X}vP66ADoCFT2g?HPvoem2eJh,zWXgd36Rb/{aRq/>7EYy6@EEH4BP(oeXac@mR')
stan_user2 = StanUser(name='IAmStanToo', fullname='Stan Robot 2', password='3X}vP66ADoCFT2g?HPvoem2eJh,zWXgd36Rb/{aRq/>7EYy6@EEH4BP(oeXac@mR')

Session = sessionmaker(bind=engine)
Session.configure(bind=engine)

sqlalchemy_url = 'postgresql://%s/%s' % (testenv['postgresql_host'], testenv['postgresql_db'])


class TestSQLAlchemy(unittest.TestCase):
    def setUp(self):
        """ Clear all spans before a test run """
        self.recorder = tracer.recorder
        self.recorder.clear_spans()
        self.session = Session()

    def tearDown(self):
        """ Ensure that allow_exit_as_root has the default value """
        agent.options.allow_exit_as_root = False

    def test_session_add(self):
        with tracer.start_active_span('test'):
            self.session.add(stan_user)
            self.session.commit()

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        sql_span = spans[0]
        test_span = spans[1]

        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, sql_span.t)

        # Parent relationships
        self.assertEqual(sql_span.p, test_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(sql_span.ec)

        # SQLAlchemy span
        self.assertEqual('sqlalchemy', sql_span.n)
        self.assertFalse('custom' in sql_span.data)
        self.assertTrue('sqlalchemy' in sql_span.data)

        self.assertEqual('postgresql', sql_span.data["sqlalchemy"]["eng"])
        self.assertEqual(sqlalchemy_url, sql_span.data["sqlalchemy"]["url"])
        self.assertEqual('INSERT INTO churchofstan (name, fullname, password) VALUES (%(name)s, %(fullname)s, %(password)s) RETURNING churchofstan.id', sql_span.data["sqlalchemy"]["sql"])
        self.assertIsNone(sql_span.data["sqlalchemy"]["err"])

        self.assertIsNotNone(sql_span.stack)
        self.assertTrue(type(sql_span.stack) is list)
        self.assertGreater(len(sql_span.stack), 0)

    def test_session_add_as_root_exit_span(self):
        agent.options.allow_exit_as_root = True
        self.session.add(stan_user2)
        self.session.commit()

        spans = self.recorder.queued_spans()
        self.assertEqual(1, len(spans))

        sql_span = spans[0]

        self.assertIsNone(tracer.active_span)

        # Parent relationships
        self.assertEqual(sql_span.p, None)

        # Error logging
        self.assertIsNone(sql_span.ec)

        # SQLAlchemy span
        self.assertEqual('sqlalchemy', sql_span.n)
        self.assertFalse('custom' in sql_span.data)
        self.assertTrue('sqlalchemy' in sql_span.data)

        self.assertEqual('postgresql', sql_span.data["sqlalchemy"]["eng"])
        self.assertEqual(sqlalchemy_url, sql_span.data["sqlalchemy"]["url"])
        self.assertEqual('INSERT INTO churchofstan (name, fullname, password) VALUES (%(name)s, %(fullname)s, %(password)s) RETURNING churchofstan.id', sql_span.data["sqlalchemy"]["sql"])
        self.assertIsNone(sql_span.data["sqlalchemy"]["err"])

        self.assertIsNotNone(sql_span.stack)
        self.assertTrue(type(sql_span.stack) is list)
        self.assertGreater(len(sql_span.stack), 0)

    def test_transaction(self):
        result = None
        with tracer.start_active_span('test'):
            with engine.begin() as connection:
                result = connection.execute(text("select 1"))
                result = connection.execute(text("select (name, fullname, password) from churchofstan where name='doesntexist'"))

        spans = self.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        sql_span0 = spans[0]
        sql_span1 = spans[1]
        test_span = spans[2]

        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, sql_span0.t)
        self.assertEqual(test_span.t, sql_span1.t)

        # Parent relationships
        self.assertEqual(sql_span0.p, test_span.s)
        self.assertEqual(sql_span1.p, test_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIsNone(sql_span0.ec)
        self.assertIsNone(sql_span1.ec)

        # SQLAlchemy span0
        self.assertEqual('sqlalchemy', sql_span0.n)
        self.assertFalse('custom' in sql_span0.data)
        self.assertTrue('sqlalchemy' in sql_span0.data)

        self.assertEqual('postgresql', sql_span0.data["sqlalchemy"]["eng"])
        self.assertEqual(sqlalchemy_url, sql_span0.data["sqlalchemy"]["url"])
        self.assertEqual('select 1', sql_span0.data["sqlalchemy"]["sql"])
        self.assertIsNone(sql_span0.data["sqlalchemy"]["err"])

        self.assertIsNotNone(sql_span0.stack)
        self.assertTrue(type(sql_span0.stack) is list)
        self.assertGreater(len(sql_span0.stack), 0)

        # SQLAlchemy span1
        self.assertEqual('sqlalchemy', sql_span1.n)
        self.assertFalse('custom' in sql_span1.data)
        self.assertTrue('sqlalchemy' in sql_span1.data)

        self.assertEqual('postgresql', sql_span1.data["sqlalchemy"]["eng"])
        self.assertEqual(sqlalchemy_url, sql_span1.data["sqlalchemy"]["url"])
        self.assertEqual("select (name, fullname, password) from churchofstan where name='doesntexist'", sql_span1.data["sqlalchemy"]["sql"])
        self.assertIsNone(sql_span1.data["sqlalchemy"]["err"])

        self.assertIsNotNone(sql_span1.stack)
        self.assertTrue(type(sql_span1.stack) is list)
        self.assertGreater(len(sql_span1.stack), 0)

    def test_error_logging(self):
        with tracer.start_active_span('test'):
            try:
                self.session.execute(text("htVwGrCwVThisIsInvalidSQLaw4ijXd88"))
                self.session.commit()
            except:
                pass

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        sql_span = spans[0]
        test_span = spans[1]

        self.assertIsNone(tracer.active_span)

        # Same traceId
        self.assertEqual(test_span.t, sql_span.t)

        # Parent relationships
        self.assertEqual(sql_span.p, test_span.s)

        # Error logging
        self.assertIsNone(test_span.ec)
        self.assertIs(sql_span.ec, 1)

        # SQLAlchemy span
        self.assertEqual('sqlalchemy', sql_span.n)

        self.assertFalse('custom' in sql_span.data)
        self.assertTrue('sqlalchemy' in sql_span.data)

        self.assertEqual('postgresql', sql_span.data["sqlalchemy"]["eng"])
        self.assertEqual(sqlalchemy_url, sql_span.data["sqlalchemy"]["url"])
        self.assertEqual('htVwGrCwVThisIsInvalidSQLaw4ijXd88', sql_span.data["sqlalchemy"]["sql"])
        self.assertIn('syntax error at or near "htVwGrCwVThisIsInvalidSQLaw4ijXd88', sql_span.data["sqlalchemy"]["err"])
        self.assertIsNotNone(sql_span.stack)
        self.assertTrue(type(sql_span.stack) is list)
        self.assertGreater(len(sql_span.stack), 0)

    def test_error_before_tracing(self):
        """Test the scenario, in which instana is loaded,
           but connection fails before tracing begins.
           This is typical in test container scenario,
           where it is "normal" to just start hammering a database container
           which is still starting and not ready to handle requests yet.
           In this scenario it is important that we get
           an sqlalachemy exception, and not something else
           like an AttributeError. Because testcontainer has a logic
           to retry in case of certain sqlalchemy exceptions but it
           can't handle an AttributeError."""
        # https://github.com/instana/python-sensor/issues/362

        self.assertIsNone(tracer.active_span)

        invalid_connection_url = 'postgresql://user1:pwd1@localhost:9999/mydb1'
        with self.assertRaisesRegex(
                OperationalError,
                r'\(psycopg2.OperationalError\) connection .* failed.*'
                               ) as context_manager:
            engine = create_engine(invalid_connection_url)
            with engine.connect() as connection:
                version, = connection.execute(text("select version()")).fetchone()

        the_exception = context_manager.exception
        self.assertFalse(the_exception.connection_invalidated)



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import unittest
import json

import boto3
from moto import mock_aws

from instana.singletons import tracer, agent
from ...helpers import get_first_span_by_filter

class TestLambda(unittest.TestCase):
    def setUp(self):
        """ Clear all spans before a test run """
        self.recorder = tracer.recorder
        self.recorder.clear_spans()
        self.mock = mock_aws(config={"lambda": {"use_docker": False}})
        self.mock.start()
        self.lambda_region = "us-east-1"
        self.aws_lambda = boto3.client('lambda', region_name=self.lambda_region)
        self.function_name = "myfunc"

    def tearDown(self):
        # Stop Moto after each test
        self.mock.stop()
        agent.options.allow_exit_as_root = False

    def test_lambda_invoke(self):
        with tracer.start_active_span('test'):
            result = self.aws_lambda.invoke(FunctionName=self.function_name, Payload=json.dumps({"message": "success"}))

        self.assertEqual(result["StatusCode"], 200)
        result_payload = json.loads(result["Payload"].read().decode("utf-8"))
        self.assertIn("message", result_payload)
        self.assertEqual("success", result_payload["message"])

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'Invoke')
        endpoint = f'https://lambda.{self.lambda_region}.amazonaws.com'
        self.assertEqual(boto_span.data['boto3']['ep'], endpoint)
        self.assertEqual(boto_span.data['boto3']['reg'], self.lambda_region)
        self.assertIn('FunctionName', boto_span.data['boto3']['payload'])
        self.assertEqual(boto_span.data['boto3']['payload']['FunctionName'], self.function_name)
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], f'{endpoint}:443/Invoke')

    def test_lambda_invoke_as_root_exit_span(self):
        agent.options.allow_exit_as_root = True
        result = self.aws_lambda.invoke(FunctionName=self.function_name, Payload=json.dumps({"message": "success"}))

        self.assertEqual(result["StatusCode"], 200)
        result_payload = json.loads(result["Payload"].read().decode("utf-8"))
        self.assertIn("message", result_payload)
        self.assertEqual("success", result_payload["message"])

        spans = self.recorder.queued_spans()
        self.assertEqual(1, len(spans))
        boto_span = spans[0]
        self.assertTrue(boto_span)
        self.assertEqual(boto_span.n, "boto3")
        self.assertIsNone(boto_span.p)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'Invoke')
        endpoint = f'https://lambda.{self.lambda_region}.amazonaws.com'
        self.assertEqual(boto_span.data['boto3']['ep'], endpoint)
        self.assertEqual(boto_span.data['boto3']['reg'], self.lambda_region)
        self.assertIn('FunctionName', boto_span.data['boto3']['payload'])
        self.assertEqual(boto_span.data['boto3']['payload']['FunctionName'], self.function_name)
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], f'{endpoint}:443/Invoke')

    def test_request_header_capture_before_call(self):
        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Capture-This', 'X-Capture-That']

        # Access the event system on the S3 client
        event_system = self.aws_lambda.meta.events

        request_headers = {
                'X-Capture-This': 'this',
                'X-Capture-That': 'that'
            }

        # Create a function that adds custom headers
        def add_custom_header_before_call(params, **kwargs):
            params['headers'].update(request_headers)

        # Register the function to before-call event.
        event_system.register('before-call.lambda.Invoke', add_custom_header_before_call)

        with tracer.start_active_span('test'):
            result = self.aws_lambda.invoke(FunctionName=self.function_name, Payload=json.dumps({"message": "success"}))

        self.assertEqual(result["StatusCode"], 200)
        result_payload = json.loads(result["Payload"].read().decode("utf-8"))
        self.assertIn("message", result_payload)
        self.assertEqual("success", result_payload["message"])

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'Invoke')
        endpoint = f'https://lambda.{self.lambda_region}.amazonaws.com'
        self.assertEqual(boto_span.data['boto3']['ep'], endpoint)
        self.assertEqual(boto_span.data['boto3']['reg'], self.lambda_region)
        self.assertIn('FunctionName', boto_span.data['boto3']['payload'])
        self.assertEqual(boto_span.data['boto3']['payload']['FunctionName'], self.function_name)
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], f'{endpoint}:443/Invoke')

        self.assertIn("X-Capture-This", boto_span.data["http"]["header"])
        self.assertEqual("this", boto_span.data["http"]["header"]["X-Capture-This"])
        self.assertIn("X-Capture-That", boto_span.data["http"]["header"])
        self.assertEqual("that", boto_span.data["http"]["header"]["X-Capture-That"])

        agent.options.extra_http_headers = original_extra_http_headers


    def test_request_header_capture_before_sign(self):
        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Custom-1', 'X-Custom-2']

        # Access the event system on the S3 client
        event_system = self.aws_lambda.meta.events

        request_headers = {
                'X-Custom-1': 'Value1',
                'X-Custom-2': 'Value2'
            }

        # Create a function that adds custom headers
        def add_custom_header_before_sign(request, **kwargs):
            for name, value in request_headers.items():
                request.headers.add_header(name, value)

        # Register the function to before-sign event.
        event_system.register_first('before-sign.lambda.Invoke', add_custom_header_before_sign)

        with tracer.start_active_span('test'):
            result = self.aws_lambda.invoke(FunctionName=self.function_name, Payload=json.dumps({"message": "success"}))

        self.assertEqual(result["StatusCode"], 200)
        result_payload = json.loads(result["Payload"].read().decode("utf-8"))
        self.assertIn("message", result_payload)
        self.assertEqual("success", result_payload["message"])

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'Invoke')
        endpoint = f'https://lambda.{self.lambda_region}.amazonaws.com'
        self.assertEqual(boto_span.data['boto3']['ep'], endpoint)
        self.assertEqual(boto_span.data['boto3']['reg'], self.lambda_region)
        self.assertIn('FunctionName', boto_span.data['boto3']['payload'])
        self.assertEqual(boto_span.data['boto3']['payload']['FunctionName'], self.function_name)
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], f'{endpoint}:443/Invoke')

        self.assertIn("X-Custom-1", boto_span.data["http"]["header"])
        self.assertEqual("Value1", boto_span.data["http"]["header"]["X-Custom-1"])
        self.assertIn("X-Custom-2", boto_span.data["http"]["header"])
        self.assertEqual("Value2", boto_span.data["http"]["header"]["X-Custom-2"])

        agent.options.extra_http_headers = original_extra_http_headers


    def test_response_header_capture(self):
        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Capture-This-Too', 'X-Capture-That-Too']

        # Access the event system on the S3 client
        event_system = self.aws_lambda.meta.events

        response_headers = {
            "X-Capture-This-Too": "this too",
            "X-Capture-That-Too": "that too",
        }

        # Create a function that sets the custom headers in the after-call event.
        def modify_after_call_args(parsed, **kwargs):
            parsed['ResponseMetadata']['HTTPHeaders'].update(response_headers)

        # Register the function to an event
        event_system.register('after-call.lambda.Invoke', modify_after_call_args)

        with tracer.start_active_span('test'):
            result = self.aws_lambda.invoke(FunctionName=self.function_name, Payload=json.dumps({"message": "success"}))

        self.assertEqual(result["StatusCode"], 200)
        result_payload = json.loads(result["Payload"].read().decode("utf-8"))
        self.assertIn("message", result_payload)
        self.assertEqual("success", result_payload["message"])

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'Invoke')
        endpoint = f'https://lambda.{self.lambda_region}.amazonaws.com'
        self.assertEqual(boto_span.data['boto3']['ep'], endpoint)
        self.assertEqual(boto_span.data['boto3']['reg'], self.lambda_region)
        self.assertIn('FunctionName', boto_span.data['boto3']['payload'])
        self.assertEqual(boto_span.data['boto3']['payload']['FunctionName'], self.function_name)
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], f'{endpoint}:443/Invoke')

        self.assertIn("X-Capture-This-Too", boto_span.data["http"]["header"])
        self.assertEqual("this too", boto_span.data["http"]["header"]["X-Capture-This-Too"])
        self.assertIn("X-Capture-That-Too", boto_span.data["http"]["header"])
        self.assertEqual("that too", boto_span.data["http"]["header"]["X-Capture-That-Too"])

        agent.options.extra_http_headers = original_extra_http_headers



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import inspect
import unittest

import opentracing as ot

import instana.propagators.http_propagator as ihp
import instana.propagators.text_propagator as itp
import instana.propagators.binary_propagator as ibp
from instana.span_context import SpanContext
from instana.tracer import InstanaTracer


class TestOTSpan(unittest.TestCase):
    def test_http_basics(self):
        inspect.isclass(ihp.HTTPPropagator)

        inject_func = getattr(ihp.HTTPPropagator, "inject", None)
        self.assertTrue(inject_func)
        self.assertTrue(callable(inject_func))

        extract_func = getattr(ihp.HTTPPropagator, "extract", None)
        self.assertTrue(extract_func)
        self.assertTrue(callable(extract_func))


    def test_http_inject_with_dict(self):
        ot.tracer = InstanaTracer()

        carrier = {}
        span = ot.tracer.start_span("unittest")
        ot.tracer.inject(span.context, ot.Format.HTTP_HEADERS, carrier)

        self.assertIn('X-INSTANA-T', carrier)
        self.assertEqual(carrier['X-INSTANA-T'], span.context.trace_id)
        self.assertIn('X-INSTANA-S', carrier)
        self.assertEqual(carrier['X-INSTANA-S'], span.context.span_id)
        self.assertIn('X-INSTANA-L', carrier)
        self.assertEqual(carrier['X-INSTANA-L'], "1")


    def test_http_inject_with_list(self):
        ot.tracer = InstanaTracer()

        carrier = []
        span = ot.tracer.start_span("unittest")
        ot.tracer.inject(span.context, ot.Format.HTTP_HEADERS, carrier)

        self.assertIn(('X-INSTANA-T', span.context.trace_id), carrier)
        self.assertIn(('X-INSTANA-S', span.context.span_id), carrier)
        self.assertIn(('X-INSTANA-L', "1"), carrier)


    def test_http_basic_extract(self):
        ot.tracer = InstanaTracer()

        carrier = {'X-INSTANA-T': '1', 'X-INSTANA-S': '1', 'X-INSTANA-L': '1', 'X-INSTANA-SYNTHETIC': '1'}
        ctx = ot.tracer.extract(ot.Format.HTTP_HEADERS, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, '0000000000000001')
        self.assertEqual(ctx.span_id, '0000000000000001')
        self.assertTrue(ctx.synthetic)


    def test_http_extract_with_byte_keys(self):
        ot.tracer = InstanaTracer()

        carrier = {b'X-INSTANA-T': '1', b'X-INSTANA-S': '1', b'X-INSTANA-L': '1', b'X-INSTANA-SYNTHETIC': '1'}
        ctx = ot.tracer.extract(ot.Format.HTTP_HEADERS, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, '0000000000000001')
        self.assertEqual(ctx.span_id, '0000000000000001')
        self.assertTrue(ctx.synthetic)


    def test_http_extract_from_list_of_tuples(self):
        ot.tracer = InstanaTracer()

        carrier = [(b'user-agent', b'python-requests/2.23.0'), (b'accept-encoding', b'gzip, deflate'),
                   (b'accept', b'*/*'), (b'connection', b'keep-alive'),
                   (b'x-instana-t', b'1'), (b'x-instana-s', b'1'), (b'x-instana-l', b'1'), (b'X-INSTANA-SYNTHETIC', '1')]
        ctx = ot.tracer.extract(ot.Format.HTTP_HEADERS, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, '0000000000000001')
        self.assertEqual(ctx.span_id, '0000000000000001')
        self.assertTrue(ctx.synthetic)


    def test_http_mixed_case_extract(self):
        ot.tracer = InstanaTracer()

        carrier = {'x-insTana-T': '1', 'X-inSTANa-S': '1', 'X-INstana-l': '1'}
        ctx = ot.tracer.extract(ot.Format.HTTP_HEADERS, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, '0000000000000001')
        self.assertEqual(ctx.span_id, '0000000000000001')
        self.assertFalse(ctx.synthetic)


    def test_http_extract_synthetic_only(self):
        ot.tracer = InstanaTracer()

        carrier = {'X-INSTANA-SYNTHETIC': '1'}
        ctx = ot.tracer.extract(ot.Format.HTTP_HEADERS, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertIsNone(ctx.trace_id)
        self.assertIsNone(ctx.span_id)
        self.assertTrue(ctx.synthetic)


    def test_http_default_context_extract(self):
        ot.tracer = InstanaTracer()

        carrier = {}
        ctx = ot.tracer.extract(ot.Format.HTTP_HEADERS, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertIsNone(ctx.trace_id)
        self.assertIsNone(ctx.span_id)
        self.assertFalse(ctx.synthetic)

    def test_http_128bit_headers(self):
        ot.tracer = InstanaTracer()

        carrier = {'X-INSTANA-T': '0000000000000000b0789916ff8f319f',
                   'X-INSTANA-S': '0000000000000000b0789916ff8f319f', 'X-INSTANA-L': '1'}
        ctx = ot.tracer.extract(ot.Format.HTTP_HEADERS, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, 'b0789916ff8f319f')
        self.assertEqual(ctx.span_id, 'b0789916ff8f319f')


    def test_text_basics(self):
        inspect.isclass(itp.TextPropagator)

        inject_func = getattr(itp.TextPropagator, "inject", None)
        self.assertTrue(inject_func)
        self.assertTrue(callable(inject_func))

        extract_func = getattr(itp.TextPropagator, "extract", None)
        self.assertTrue(extract_func)
        self.assertTrue(callable(extract_func))


    def test_text_inject_with_dict(self):
        ot.tracer = InstanaTracer()

        carrier = {}
        span = ot.tracer.start_span("unittest")
        ot.tracer.inject(span.context, ot.Format.TEXT_MAP, carrier)

        self.assertIn('x-instana-t', carrier)
        self.assertEqual(carrier['x-instana-t'], span.context.trace_id)
        self.assertIn('x-instana-s', carrier)
        self.assertEqual(carrier['x-instana-s'], span.context.span_id)
        self.assertIn('x-instana-l', carrier)
        self.assertEqual(carrier['x-instana-l'], "1")


    def test_text_inject_with_list(self):
        ot.tracer = InstanaTracer()

        carrier = []
        span = ot.tracer.start_span("unittest")
        ot.tracer.inject(span.context, ot.Format.TEXT_MAP, carrier)

        self.assertIn(('x-instana-t', span.context.trace_id), carrier)
        self.assertIn(('x-instana-s', span.context.span_id), carrier)
        self.assertIn(('x-instana-l', "1"), carrier)


    def test_text_basic_extract(self):
        ot.tracer = InstanaTracer()

        carrier = {'x-instana-t': '1', 'x-instana-s': '1', 'x-instana-l': '1'}
        ctx = ot.tracer.extract(ot.Format.TEXT_MAP, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, '0000000000000001')
        self.assertEqual(ctx.span_id, '0000000000000001')


    def test_text_mixed_case_extract(self):
        ot.tracer = InstanaTracer()

        carrier = {'x-insTana-T': '1', 'X-inSTANa-S': '1', 'X-INstana-l': '1'}
        ctx = ot.tracer.extract(ot.Format.TEXT_MAP, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, '0000000000000001')
        self.assertEqual(ctx.span_id, '0000000000000001')


    def test_text_default_context_extract(self):
        ot.tracer = InstanaTracer()

        carrier = {}
        ctx = ot.tracer.extract(ot.Format.TEXT_MAP, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertIsNone(ctx.trace_id)
        self.assertIsNone(ctx.span_id)
        self.assertFalse(ctx.synthetic)


    def test_text_128bit_headers(self):
        ot.tracer = InstanaTracer()

        carrier = {'x-instana-t': '0000000000000000b0789916ff8f319f',
                   'x-instana-s': ' 0000000000000000b0789916ff8f319f', 'X-INSTANA-L': '1'}
        ctx = ot.tracer.extract(ot.Format.TEXT_MAP, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, 'b0789916ff8f319f')
        self.assertEqual(ctx.span_id, 'b0789916ff8f319f')

    def test_binary_basics(self):
        inspect.isclass(ibp.BinaryPropagator)

        inject_func = getattr(ibp.BinaryPropagator, "inject", None)
        self.assertTrue(inject_func)
        self.assertTrue(callable(inject_func))

        extract_func = getattr(ibp.BinaryPropagator, "extract", None)
        self.assertTrue(extract_func)
        self.assertTrue(callable(extract_func))


    def test_binary_inject_with_dict(self):
        ot.tracer = InstanaTracer()

        carrier = {}
        span = ot.tracer.start_span("unittest")
        ot.tracer.inject(span.context, ot.Format.BINARY, carrier)

        self.assertIn(b'x-instana-t', carrier)
        self.assertEqual(carrier[b'x-instana-t'], str.encode(span.context.trace_id))
        self.assertIn(b'x-instana-s', carrier)
        self.assertEqual(carrier[b'x-instana-s'], str.encode(span.context.span_id))
        self.assertIn(b'x-instana-l', carrier)
        self.assertEqual(carrier[b'x-instana-l'], b'1')


    def test_binary_inject_with_list(self):
        ot.tracer = InstanaTracer()

        carrier = []
        span = ot.tracer.start_span("unittest")
        ot.tracer.inject(span.context, ot.Format.BINARY, carrier)

        self.assertIn((b'x-instana-t', str.encode(span.context.trace_id)), carrier)
        self.assertIn((b'x-instana-s', str.encode(span.context.span_id)), carrier)
        self.assertIn((b'x-instana-l', b'1'), carrier)


    def test_binary_basic_extract(self):
        ot.tracer = InstanaTracer()

        carrier = {b'X-INSTANA-T': b'1', b'X-INSTANA-S': b'1', b'X-INSTANA-L': b'1', b'X-INSTANA-SYNTHETIC': b'1'}
        ctx = ot.tracer.extract(ot.Format.BINARY, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, '0000000000000001')
        self.assertEqual(ctx.span_id, '0000000000000001')
        self.assertTrue(ctx.synthetic)


    def test_binary_mixed_case_extract(self):
        ot.tracer = InstanaTracer()

        carrier = {'x-insTana-T': '1', 'X-inSTANa-S': '1', 'X-INstana-l': '1', b'X-inStaNa-SYNtheTIC': b'1'}
        ctx = ot.tracer.extract(ot.Format.BINARY, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, '0000000000000001')
        self.assertEqual(ctx.span_id, '0000000000000001')
        self.assertTrue(ctx.synthetic)


    def test_binary_default_context_extract(self):
        ot.tracer = InstanaTracer()

        carrier = {}
        ctx = ot.tracer.extract(ot.Format.BINARY, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertIsNone(ctx.trace_id)
        self.assertIsNone(ctx.span_id)
        self.assertFalse(ctx.synthetic)


    def test_binary_128bit_headers(self):
        ot.tracer = InstanaTracer()

        carrier = {'X-INSTANA-T': '0000000000000000b0789916ff8f319f',
                   'X-INSTANA-S': ' 0000000000000000b0789916ff8f319f', 'X-INSTANA-L': '1'}
        ctx = ot.tracer.extract(ot.Format.BINARY, carrier)

        self.assertIsInstance(ctx, SpanContext)
        self.assertEqual(ctx.trace_id, 'b0789916ff8f319f')
        self.assertEqual(ctx.span_id, 'b0789916ff8f319f')



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

"""
Celery Signals are disjointed and don't allow us to pass the scope object along
with the Job message so we instead store all scopes in a dictionary on the
registered Task job.

These methods allow pushing and pop'ing of scopes on Task objects.

WeakValueDictionary allows for lost scopes to be garbage collected.
"""

from weakref import WeakValueDictionary


def get_task_id(headers, body):
    """
    Across Celery versions, the task id can exist in a couple of places.
    """
    id = headers.get('id', None)
    if id is None:
        id = body.get('id', None)
    return id


def task_catalog_push(task, task_id, scope, is_consumer):
    """
    Push (adds) an object to the task catalog
    @param task: The Celery Task
    @param task_id:  The Celery Task ID
    @param is_consumer: Boolean
    @return: scope
    """
    catalog = None
    if not hasattr(task, '_instana_scopes'):
        catalog = WeakValueDictionary()
        setattr(task, '_instana_scopes', catalog)
    else:
        catalog = getattr(task, '_instana_scopes')

    key = (task_id, is_consumer)
    catalog[key] = scope


def task_catalog_pop(task, task_id, is_consumer):
    """
    Pop (removes) an object from the task catalog
    @param task: The Celery Task
    @param task_id:  The Celery Task ID
    @param is_consumer: Boolean
    @return: scope
    """
    catalog = getattr(task, '_instana_scopes', None)
    if catalog is None:
        return None

    key = (task_id, is_consumer)
    return catalog.pop(key, None)


def task_catalog_get(task, task_id, is_consumer):
    """
    Get an object from the task catalog
    @param task: The Celery Task
    @param task_id:  The Celery Task ID
    @param is_consumer: Boolean
    @return: scope
    """
    catalog = getattr(task, '_instana_scopes', None)
    if catalog is None:
        return None

    key = (task_id, is_consumer)
    return catalog.get(key, None)




# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import time
import unittest
import random
import threading
import sys
import traceback

from instana.autoprofile.profiler import Profiler
from instana.autoprofile.runtime import runtime_info
from instana.autoprofile.samplers.cpu_sampler import CPUSampler


class CPUSamplerTestCase(unittest.TestCase):

    def test_cpu_profile(self):
        if runtime_info.OS_WIN:
            return

        profiler = Profiler(None)
        profiler.start(disable_timers=True)
        sampler = CPUSampler(profiler)
        sampler.setup()
        sampler.reset()

        def record():
            sampler.start_sampler()
            time.sleep(2)
            sampler.stop_sampler()

        record_t = threading.Thread(target=record)
        record_t.start()

        def cpu_work_main_thread():
            for i in range(0, 1000000):
                text = "text1" + str(i)
                text = text + "text2"
        cpu_work_main_thread()

        record_t.join()

        profile = sampler.build_profile(2000, 120000).to_dict()
        #print(profile)

        self.assertTrue('cpu_work_main_thread' in str(profile))


if __name__ == '__main__':
    unittest.main()



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import logging
import unittest

import MySQLdb

from ..helpers import testenv
from instana.singletons import  agent, tracer

logger = logging.getLogger(__name__)


class TestMySQLPython(unittest.TestCase):
    def setUp(self):
        self.db = MySQLdb.connect(host=testenv['mysql_host'], port=testenv['mysql_port'],
                                  user=testenv['mysql_user'], passwd=testenv['mysql_pw'],
                                  db=testenv['mysql_db'])
        database_setup_query = """
        DROP TABLE IF EXISTS users;
        CREATE TABLE users(
            id serial primary key,
            name varchar(40) NOT NULL,
            email varchar(40) NOT NULL
        );
        INSERT INTO users(name, email) VALUES('kermit', 'kermit@muppets.com');
        DROP PROCEDURE IF EXISTS test_proc;
        CREATE PROCEDURE test_proc(IN t VARCHAR(255))
        BEGIN
            SELECT name FROM users WHERE name = t;
        END
        """
        setup_cursor = self.db.cursor()
        setup_cursor.execute(database_setup_query)
        setup_cursor.close()

        self.cursor = self.db.cursor()
        self.recorder = tracer.recorder
        self.recorder.clear_spans()
        tracer.cur_ctx = None

    def tearDown(self):
        if self.cursor and self.cursor.connection.open:
          self.cursor.close()
        if self.db and self.db.open:
          self.db.close()
        agent.options.allow_exit_as_root = False

    def test_vanilla_query(self):
        affected_rows = self.cursor.execute("""SELECT * from users""")
        self.assertEqual(1, affected_rows)
        result = self.cursor.fetchone()
        self.assertEqual(3, len(result))

        spans = self.recorder.queued_spans()
        self.assertEqual(0, len(spans))

    def test_basic_query(self):
        with tracer.start_active_span('test'):
            affected_rows = self.cursor.execute("""SELECT * from users""")
            result = self.cursor.fetchone()

        self.assertEqual(1, affected_rows)
        self.assertEqual(3, len(result))

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mysql")
        self.assertEqual(db_span.data["mysql"]["db"], testenv['mysql_db'])
        self.assertEqual(db_span.data["mysql"]["user"], testenv['mysql_user'])
        self.assertEqual(db_span.data["mysql"]["stmt"], 'SELECT * from users')
        self.assertEqual(db_span.data["mysql"]["host"], testenv['mysql_host'])
        self.assertEqual(db_span.data["mysql"]["port"], testenv['mysql_port'])

    def test_basic_query_as_root_exit_span(self):
        agent.options.allow_exit_as_root = True
        affected_rows = self.cursor.execute("""SELECT * from users""")
        result = self.cursor.fetchone()

        self.assertEqual(1, affected_rows)
        self.assertEqual(3, len(result))

        spans = self.recorder.queued_spans()
        self.assertEqual(1, len(spans))

        db_span = spans[0]

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mysql")
        self.assertEqual(db_span.data["mysql"]["db"], testenv['mysql_db'])
        self.assertEqual(db_span.data["mysql"]["user"], testenv['mysql_user'])
        self.assertEqual(db_span.data["mysql"]["stmt"], 'SELECT * from users')
        self.assertEqual(db_span.data["mysql"]["host"], testenv['mysql_host'])
        self.assertEqual(db_span.data["mysql"]["port"], testenv['mysql_port'])

    def test_basic_insert(self):
        with tracer.start_active_span('test'):
            affected_rows = self.cursor.execute(
                        """INSERT INTO users(name, email) VALUES(%s, %s)""",
                        ('beaker', 'beaker@muppets.com'))

        self.assertEqual(1, affected_rows)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mysql")
        self.assertEqual(db_span.data["mysql"]["db"], testenv['mysql_db'])
        self.assertEqual(db_span.data["mysql"]["user"], testenv['mysql_user'])
        self.assertEqual(db_span.data["mysql"]["stmt"], 'INSERT INTO users(name, email) VALUES(%s, %s)')
        self.assertEqual(db_span.data["mysql"]["host"], testenv['mysql_host'])
        self.assertEqual(db_span.data["mysql"]["port"], testenv['mysql_port'])

    def test_executemany(self):
        with tracer.start_active_span('test'):
            affected_rows = self.cursor.executemany("INSERT INTO users(name, email) VALUES(%s, %s)",
                                             [('beaker', 'beaker@muppets.com'), ('beaker', 'beaker@muppets.com')])
            self.db.commit()

        self.assertEqual(2, affected_rows)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mysql")
        self.assertEqual(db_span.data["mysql"]["db"], testenv['mysql_db'])
        self.assertEqual(db_span.data["mysql"]["user"], testenv['mysql_user'])
        self.assertEqual(db_span.data["mysql"]["stmt"], 'INSERT INTO users(name, email) VALUES(%s, %s)')
        self.assertEqual(db_span.data["mysql"]["host"], testenv['mysql_host'])
        self.assertEqual(db_span.data["mysql"]["port"], testenv['mysql_port'])

    def test_call_proc(self):
        with tracer.start_active_span('test'):
            callproc_result = self.cursor.callproc('test_proc', ('beaker',))

        self.assertIsInstance(callproc_result, tuple)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mysql")
        self.assertEqual(db_span.data["mysql"]["db"], testenv['mysql_db'])
        self.assertEqual(db_span.data["mysql"]["user"], testenv['mysql_user'])
        self.assertEqual(db_span.data["mysql"]["stmt"], 'test_proc')
        self.assertEqual(db_span.data["mysql"]["host"], testenv['mysql_host'])
        self.assertEqual(db_span.data["mysql"]["port"], testenv['mysql_port'])

    def test_error_capture(self):
        affected_rows = None
        try:
            with tracer.start_active_span('test'):
                affected_rows = self.cursor.execute("""SELECT * from blah""")
        except Exception:
            pass

        self.assertIsNone(affected_rows)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertEqual(1, db_span.ec)
        self.assertEqual(db_span.data["mysql"]["error"], '(1146, "Table \'%s.blah\' doesn\'t exist")' % testenv['mysql_db'])

        self.assertEqual(db_span.n, "mysql")
        self.assertEqual(db_span.data["mysql"]["db"], testenv['mysql_db'])
        self.assertEqual(db_span.data["mysql"]["user"], testenv['mysql_user'])
        self.assertEqual(db_span.data["mysql"]["stmt"], 'SELECT * from blah')
        self.assertEqual(db_span.data["mysql"]["host"], testenv['mysql_host'])
        self.assertEqual(db_span.data["mysql"]["port"], testenv['mysql_port'])

    def test_connect_cursor_ctx_mgr(self):
        with tracer.start_active_span("test"):
            with self.db as connection:
                with connection.cursor() as cursor:
                    affected_rows = cursor.execute("""SELECT * from users""")

        self.assertEqual(1, affected_rows)
        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mysql")
        self.assertEqual(db_span.data["mysql"]["db"], testenv["mysql_db"])
        self.assertEqual(db_span.data["mysql"]["user"], testenv["mysql_user"])
        self.assertEqual(db_span.data["mysql"]["stmt"], "SELECT * from users")
        self.assertEqual(db_span.data["mysql"]["host"], testenv["mysql_host"])
        self.assertEqual(db_span.data["mysql"]["port"], testenv["mysql_port"])

    def test_connect_ctx_mgr(self):
        with tracer.start_active_span("test"):
            with self.db as connection:
                cursor = connection.cursor()
                cursor.execute("""SELECT * from users""")


        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mysql")
        self.assertEqual(db_span.data["mysql"]["db"], testenv["mysql_db"])
        self.assertEqual(db_span.data["mysql"]["user"], testenv["mysql_user"])
        self.assertEqual(db_span.data["mysql"]["stmt"], "SELECT * from users")
        self.assertEqual(db_span.data["mysql"]["host"], testenv["mysql_host"])
        self.assertEqual(db_span.data["mysql"]["port"], testenv["mysql_port"])

    def test_cursor_ctx_mgr(self):
        with tracer.start_active_span("test"):
            connection = self.db
            with connection.cursor() as cursor:
                affected_rows = cursor.execute("""SELECT * from users""")


        self.assertEqual(1, affected_rows)
        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mysql")
        self.assertEqual(db_span.data["mysql"]["db"], testenv["mysql_db"])
        self.assertEqual(db_span.data["mysql"]["user"], testenv["mysql_user"])
        self.assertEqual(db_span.data["mysql"]["stmt"], "SELECT * from users")
        self.assertEqual(db_span.data["mysql"]["host"], testenv["mysql_host"])
        self.assertEqual(db_span.data["mysql"]["port"], testenv["mysql_port"])



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import time
import unittest
import random
import threading

from instana.autoprofile.profiler import Profiler
from instana.autoprofile.runtime import min_version, runtime_info
from instana.autoprofile.samplers.allocation_sampler import AllocationSampler


class AllocationSamplerTestCase(unittest.TestCase):

    def test_allocation_profile(self):
        if runtime_info.OS_WIN or not min_version(3, 4):
            return

        profiler = Profiler(None)
        profiler.start(disable_timers=True)
        sampler = AllocationSampler(profiler)
        sampler.setup()
        sampler.reset()

        mem1 = []
        def mem_leak(n = 100000):
            mem2 = []
            for i in range(0, n):
                mem1.append(random.randint(0, 1000))
                mem2.append(random.randint(0, 1000))

        def mem_leak2():
            mem_leak()

        def mem_leak3():
            mem_leak2()

        def mem_leak4():
            mem_leak3()

        def mem_leak5():
            mem_leak4()

        def record():
            sampler.start_sampler()
            time.sleep(2)
            sampler.stop_sampler()

        t = threading.Thread(target=record)
        t.start()

        # simulate leak
        mem_leak5()

        t.join()

        profile = sampler.build_profile(2000, 120000).to_dict()
        #print(profile)

        self.assertTrue('test_allocation_sampler.py' in str(profile))


if __name__ == '__main__':
    unittest.main()



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import os
import json
import unittest

from instana.tracer import InstanaTracer
from instana.recorder import StanRecorder
from instana.agent.aws_fargate import AWSFargateAgent
from instana.singletons import get_agent, set_agent, get_tracer, set_tracer


def get_docker_plugin(plugins):
    """
    Given a list of plugins, find and return the docker plugin that we're interested in from the mock data
    """
    docker_plugin = None
    for plugin in plugins:
        if plugin["name"] == "com.instana.plugin.docker" and plugin["entityId"] == "arn:aws:ecs:us-east-2:410797082306:task/2d60afb1-e7fd-4761-9430-a375293a9b82::docker-ssh-aws-fargate":
            docker_plugin = plugin
    return docker_plugin


class TestFargateCollector(unittest.TestCase):
    def __init__(self, methodName='runTest'):
        super(TestFargateCollector, self).__init__(methodName)
        self.agent = None
        self.span_recorder = None
        self.tracer = None
        self.pwd = os.path.dirname(os.path.realpath(__file__))

        self.original_agent = get_agent()
        self.original_tracer = get_tracer()

    def setUp(self):
        os.environ["AWS_EXECUTION_ENV"] = "AWS_ECS_FARGATE"
        os.environ["INSTANA_ENDPOINT_URL"] = "https://localhost/notreal"
        os.environ["INSTANA_AGENT_KEY"] = "Fake_Key"

        if "INSTANA_ZONE" in os.environ:
            os.environ.pop("INSTANA_ZONE")
        if "INSTANA_TAGS" in os.environ:
            os.environ.pop("INSTANA_TAGS")

    def tearDown(self):
        """ Reset all environment variables of consequence """
        if "AWS_EXECUTION_ENV" in os.environ:
            os.environ.pop("AWS_EXECUTION_ENV")
        if "INSTANA_EXTRA_HTTP_HEADERS" in os.environ:
            os.environ.pop("INSTANA_EXTRA_HTTP_HEADERS")
        if "INSTANA_ENDPOINT_URL" in os.environ:
            os.environ.pop("INSTANA_ENDPOINT_URL")
        if "INSTANA_AGENT_KEY" in os.environ:
            os.environ.pop("INSTANA_AGENT_KEY")
        if "INSTANA_ZONE" in os.environ:
            os.environ.pop("INSTANA_ZONE")
        if "INSTANA_TAGS" in os.environ:
            os.environ.pop("INSTANA_TAGS")

        set_agent(self.original_agent)
        set_tracer(self.original_tracer)

    def create_agent_and_setup_tracer(self):
        self.agent = AWSFargateAgent()
        self.span_recorder = StanRecorder(self.agent)
        self.tracer = InstanaTracer(recorder=self.span_recorder)
        set_agent(self.agent)
        set_tracer(self.tracer)

        # Manually set the ECS Metadata API results on the collector
        with open(self.pwd + '/../data/fargate/1.3.0/root_metadata.json', 'r') as json_file:
            self.agent.collector.root_metadata = json.load(json_file)
        with open(self.pwd + '/../data/fargate/1.3.0/task_metadata.json', 'r') as json_file:
            self.agent.collector.task_metadata = json.load(json_file)
        with open(self.pwd + '/../data/fargate/1.3.0/stats_metadata.json', 'r') as json_file:
            self.agent.collector.stats_metadata = json.load(json_file)
        with open(self.pwd + '/../data/fargate/1.3.0/task_stats_metadata.json', 'r') as json_file:
            self.agent.collector.task_stats_metadata = json.load(json_file)

    def test_prepare_payload_basics(self):
        self.create_agent_and_setup_tracer()

        payload = self.agent.collector.prepare_payload()
        self.assertTrue(payload)

        self.assertEqual(2, len(payload.keys()))
        self.assertIn('spans',payload)
        self.assertIsInstance(payload['spans'], list)
        self.assertEqual(0, len(payload['spans']))
        self.assertIn('metrics', payload)
        self.assertEqual(1, len(payload['metrics'].keys()))
        self.assertIn('plugins', payload['metrics'])
        self.assertIsInstance(payload['metrics']['plugins'], list)
        self.assertEqual(7, len(payload['metrics']['plugins']))

        plugins = payload['metrics']['plugins']
        for plugin in plugins:
            # print("%s - %s" % (plugin["name"], plugin["entityId"]))
            self.assertIn('name', plugin)
            self.assertIn('entityId', plugin)
            self.assertIn('data', plugin)

    def test_docker_plugin_snapshot_data(self):
        self.create_agent_and_setup_tracer()

        first_payload = self.agent.collector.prepare_payload()
        second_payload = self.agent.collector.prepare_payload()

        self.assertTrue(first_payload)
        self.assertTrue(second_payload)

        plugin_first_report = get_docker_plugin(first_payload['metrics']['plugins'])
        plugin_second_report = get_docker_plugin(second_payload['metrics']['plugins'])

        self.assertTrue(plugin_first_report)
        self.assertIn("data", plugin_first_report)

        # First report should have snapshot data
        data = plugin_first_report["data"]
        self.assertEqual(data["Id"], "63dc7ac9f3130bba35c785ed90ff12aad82087b5c5a0a45a922c45a64128eb45")
        self.assertEqual(data["Created"], "2020-07-27T12:14:12.583114444Z")
        self.assertEqual(data["Started"], "2020-07-27T12:14:13.545410186Z")
        self.assertEqual(data["Image"], "410797082306.dkr.ecr.us-east-2.amazonaws.com/fargate-docker-ssh:latest")
        self.assertEqual(data["Labels"], {'com.amazonaws.ecs.cluster': 'arn:aws:ecs:us-east-2:410797082306:cluster/lombardo-ssh-cluster', 'com.amazonaws.ecs.container-name': 'docker-ssh-aws-fargate', 'com.amazonaws.ecs.task-arn': 'arn:aws:ecs:us-east-2:410797082306:task/2d60afb1-e7fd-4761-9430-a375293a9b82', 'com.amazonaws.ecs.task-definition-family': 'docker-ssh-aws-fargate', 'com.amazonaws.ecs.task-definition-version': '1'})
        self.assertIsNone(data["Ports"])

        # Second report should have no snapshot data
        self.assertTrue(plugin_second_report)
        self.assertIn("data", plugin_second_report)
        data = plugin_second_report["data"]
        self.assertIn("Id", data)
        self.assertNotIn("Created", data)
        self.assertNotIn("Started", data)
        self.assertNotIn("Image", data)
        self.assertNotIn("Labels", data)
        self.assertNotIn("Ports", data)

    def test_docker_plugin_metrics(self):
        self.create_agent_and_setup_tracer()

        first_payload = self.agent.collector.prepare_payload()
        second_payload = self.agent.collector.prepare_payload()

        self.assertTrue(first_payload)
        self.assertTrue(second_payload)

        plugin_first_report = get_docker_plugin(first_payload['metrics']['plugins'])
        self.assertTrue(plugin_first_report)
        self.assertIn("data", plugin_first_report)

        plugin_second_report = get_docker_plugin(second_payload['metrics']['plugins'])
        self.assertTrue(plugin_second_report)
        self.assertIn("data", plugin_second_report)

        # First report should report all metrics
        data = plugin_first_report.get("data", None)
        self.assertTrue(data)
        self.assertNotIn("network", data)

        cpu = data.get("cpu", None)
        self.assertTrue(cpu)
        self.assertEqual(cpu["total_usage"], 0.011033)
        self.assertEqual(cpu["user_usage"], 0.009918)
        self.assertEqual(cpu["system_usage"], 0.00089)
        self.assertEqual(cpu["throttling_count"], 0)
        self.assertEqual(cpu["throttling_time"], 0)

        memory = data.get("memory", None)
        self.assertTrue(memory)
        self.assertEqual(memory["active_anon"], 78721024)
        self.assertEqual(memory["active_file"], 18501632)
        self.assertEqual(memory["inactive_anon"], 0)
        self.assertEqual(memory["inactive_file"], 71684096)
        self.assertEqual(memory["total_cache"], 90185728)
        self.assertEqual(memory["total_rss"], 78721024)
        self.assertEqual(memory["usage"], 193769472)
        self.assertEqual(memory["max_usage"], 195305472)
        self.assertEqual(memory["limit"], 536870912)

        blkio = data.get("blkio", None)
        self.assertTrue(blkio)
        self.assertEqual(blkio["blk_read"], 0)
        self.assertEqual(blkio["blk_write"], 128352256)

        # Second report should report the delta (in the test case, nothing)
        data = plugin_second_report["data"]
        self.assertIn("cpu", data)
        self.assertEqual(len(data["cpu"]), 0)
        self.assertIn("memory", data)
        self.assertEqual(len(data["memory"]), 0)
        self.assertIn("blkio", data)
        self.assertEqual(len(data["blkio"]), 1)
        self.assertEqual(data["blkio"]['blk_write'], 0)
        self.assertNotIn('blk_read', data["blkio"])

    def test_no_instana_zone(self):
        self.create_agent_and_setup_tracer()
        self.assertIsNone(self.agent.options.zone)

    def test_instana_zone(self):
        os.environ["INSTANA_ZONE"] = "YellowDog"
        self.create_agent_and_setup_tracer()

        self.assertEqual(self.agent.options.zone, "YellowDog")

        payload = self.agent.collector.prepare_payload()
        self.assertTrue(payload)

        plugins = payload['metrics']['plugins']
        self.assertIsInstance(plugins, list)

        task_plugin = None
        for plugin in plugins:
            if plugin["name"] == "com.instana.plugin.aws.ecs.task":
                task_plugin = plugin

        self.assertTrue(task_plugin)
        self.assertIn("data", task_plugin)
        self.assertIn("instanaZone", task_plugin["data"])
        self.assertEqual(task_plugin["data"]["instanaZone"], "YellowDog")

    def test_custom_tags(self):
        os.environ["INSTANA_TAGS"] = "love,war=1,games"
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent.options, 'tags'))
        self.assertDictEqual(self.agent.options.tags, {"love": None, "war": "1", "games": None})

        payload = self.agent.collector.prepare_payload()

        self.assertTrue(payload)
        task_plugin = None
        plugins = payload['metrics']['plugins']
        for plugin in plugins:
            if plugin["name"] == "com.instana.plugin.aws.ecs.task":
                task_plugin = plugin
        self.assertTrue(task_plugin)
        self.assertIn("tags", task_plugin["data"])
        tags = task_plugin["data"]["tags"]
        self.assertEqual(tags["war"], "1")
        self.assertIsNone(tags["love"])
        self.assertIsNone(tags["games"])



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import unittest
import threading

from instana.autoprofile.profiler import Profiler
from instana.autoprofile.runtime import runtime_info, min_version


# python3 -m unittest discover -v -s tests -p *_test.py

class ProfilerTestCase(unittest.TestCase):

    def test_run_in_main_thread(self):
        if runtime_info.OS_WIN:
            return

        profiler = Profiler(None)
        profiler.start(disable_timers=True)

        result = {}

        def _run():
            result['thread_id'] = threading.current_thread().ident

        def _thread():
            profiler.run_in_main_thread(_run)

        t = threading.Thread(target=_thread)
        t.start()
        t.join()

        self.assertEqual(result['thread_id'], threading.current_thread().ident)
        

if __name__ == '__main__':
    unittest.main()



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import logging
import unittest
import pytest
from instana.singletons import agent, tracer


class TestLogging(unittest.TestCase):
    @pytest.fixture
    def capture_log(self, caplog):
        self.caplog = caplog

    def setUp(self):
        """ Clear all spans before a test run """
        self.recorder = tracer.recorder
        self.recorder.clear_spans()
        self.logger = logging.getLogger('unit test')

    def tearDown(self):
        """ Ensure that allow_exit_as_root has the default value """
        agent.options.allow_exit_as_root = False

    def test_no_span(self):
        with tracer.start_active_span('test'):
            self.logger.info('info message')

        spans = self.recorder.queued_spans()
        self.assertEqual(1, len(spans))

    def test_extra_span(self):
        with tracer.start_active_span('test'):
            self.logger.warning('foo %s', 'bar')

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))
        self.assertEqual(2, spans[0].k)

        self.assertEqual('foo bar', spans[0].data["log"].get('message'))

    def test_log_with_tuple(self):
        with tracer.start_active_span('test'):
            self.logger.warning('foo %s', ("bar",))

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))
        self.assertEqual(2, spans[0].k)

        self.assertEqual("foo ('bar',)", spans[0].data["log"].get('message'))

    def test_parameters(self):
        with tracer.start_active_span('test'):
            try:
                a = 42
                b = 0
                c = a / b
            except Exception as e:
                self.logger.exception('Exception: %s', str(e))

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        self.assertIsNotNone(spans[0].data["log"].get('parameters'))

    def test_no_root_exit_span(self):
        agent.options.allow_exit_as_root = True
        self.logger.info('info message')

        spans = self.recorder.queued_spans()
        self.assertEqual(0, len(spans))

    def test_root_exit_span(self):
        agent.options.allow_exit_as_root = True
        self.logger.warning('foo %s', 'bar')

        spans = self.recorder.queued_spans()
        self.assertEqual(1, len(spans))
        self.assertEqual(2, spans[0].k)

        self.assertEqual('foo bar', spans[0].data["log"].get('message'))

    @pytest.mark.usefixtures("capture_log")
    def test_log_caller(self):
        handler = logging.StreamHandler()
        handler.setFormatter(
            logging.Formatter("source: %(funcName)s, message: %(message)s")
        )
        self.logger.addHandler(handler)

        def log_custom_warning():
            self.logger.warning("foo %s", "bar")

        with tracer.start_active_span("test"):
            log_custom_warning()
        self.assertEqual(self.caplog.records[0].funcName, "log_custom_warning")

        self.logger.removeHandler(handler)



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2019


import sys
import wrapt
import logging
from collections.abc import Mapping

from ..log import logger
from ..util.traceutils import get_tracer_tuple, tracing_is_off


@wrapt.patch_function_wrapper('logging', 'Logger._log')
def log_with_instana(wrapped, instance, argv, kwargs):
    # argv[0] = level
    # argv[1] = message
    # argv[2] = args for message
    if sys.version_info >= (3, 13):
        stacklevel = 3
    else:
        stacklevel = 2
    try:
        tracer, parent_span, _ = get_tracer_tuple()

        # Only needed if we're tracing and serious log
        if tracing_is_off() or argv[0] < logging.WARN:
            return wrapped(*argv, **kwargs, stacklevel=stacklevel)

        msg = str(argv[1])
        args = argv[2]
        if args and len(args) == 1 and isinstance(args[0], Mapping) and args[0]:
            args = args[0]

        # get the formatted log message
        msg = msg % args

        # get additional information if an exception is being handled
        parameters = None
        (t, v, tb) = sys.exc_info()
        if t is not None and v is not None:
            parameters = '{} {}'.format(t , v)

        # create logging span
        with tracer.start_active_span('log', child_of=parent_span) as scope:
            scope.span.log_kv({ 'message': msg })
            if parameters is not None:
                scope.span.log_kv({ 'parameters': parameters })
            # extra tags for an error
            if argv[0] >= logging.ERROR:
                scope.span.mark_as_errored()
    except Exception:
        logger.debug('log_with_instana:', exc_info=True)

    return wrapped(*argv, **kwargs, stacklevel=stacklevel)


logger.debug('Instrumenting logging')




# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

"""
Instrumentation for FastAPI
https://fastapi.tiangolo.com/
"""
try:
    import fastapi
    import os
    import wrapt
    import signal

    from ..log import logger
    from ..util.gunicorn import running_in_gunicorn
    from .asgi import InstanaASGIMiddleware
    from starlette.middleware import Middleware
    from fastapi import HTTPException
    from fastapi.exception_handlers import http_exception_handler

    from instana.singletons import async_tracer

    if not(hasattr(fastapi, '__version__')
            and (fastapi.__version__[0] > '0' or
                 int(fastapi.__version__.split('.')[1]) >= 51)):
        logger.debug('Instana supports FastAPI package versions 0.51.0 and newer.  Skipping.')
        raise ImportError

    async def instana_exception_handler(request, exc):
        """
        We capture FastAPI HTTPException, log the error and pass it on
        to the default exception handler.
        """
        try:
            span = async_tracer.active_span

            if span is not None:
                if hasattr(exc, 'detail') and 500 <= exc.status_code:
                    span.set_tag('http.error', exc.detail)
                span.set_tag('http.status_code', exc.status_code)
        except Exception:
            logger.debug("FastAPI instana_exception_handler: ", exc_info=True)

        return await http_exception_handler(request, exc)

    @wrapt.patch_function_wrapper('fastapi.applications', 'FastAPI.__init__')
    def init_with_instana(wrapped, instance, args, kwargs):
        middleware = kwargs.get('middleware')
        if middleware is None:
            kwargs['middleware'] = [Middleware(InstanaASGIMiddleware)]
        elif isinstance(middleware, list):
            middleware.append(Middleware(InstanaASGIMiddleware))

        exception_handlers = kwargs.get('exception_handlers')
        if exception_handlers is None:
            kwargs['exception_handlers'] = dict()

        if isinstance(kwargs['exception_handlers'], dict):
            kwargs['exception_handlers'][HTTPException] = instana_exception_handler

        return wrapped(*args, **kwargs)

    logger.debug("Instrumenting FastAPI")

    # Reload GUnicorn when we are instrumenting an already running application
    if "INSTANA_MAGIC" in os.environ and running_in_gunicorn():
        os.kill(os.getpid(), signal.SIGHUP)

except ImportError:
    pass



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import os
import logging
import unittest

from mock import MagicMock, patch
import requests

from instana.agent.host import HostAgent
from instana.fsm import Discovery
from instana.log import logger
from instana.options import StandardOptions
from instana.recorder import StanRecorder
from instana.singletons import get_agent, set_agent, get_tracer, set_tracer
from instana.tracer import InstanaTracer


class TestHost(unittest.TestCase):
    def __init__(self, methodName='runTest'):
        super(TestHost, self).__init__(methodName)
        self.agent = None
        self.span_recorder = None
        self.tracer = None

        self.original_agent = get_agent()
        self.original_tracer = get_tracer()

    def setUp(self):
        pass

    def tearDown(self):
        """ Reset all environment variables of consequence """
        variable_names = (
                "AWS_EXECUTION_ENV", "INSTANA_EXTRA_HTTP_HEADERS",
                "INSTANA_ENDPOINT_URL", "INSTANA_ENDPOINT_PROXY",
                "INSTANA_AGENT_KEY", "INSTANA_LOG_LEVEL",
                "INSTANA_SERVICE_NAME", "INSTANA_SECRETS", "INSTANA_TAGS",
                )

        for variable_name in variable_names:
            if variable_name in os.environ:
                os.environ.pop(variable_name)

        set_agent(self.original_agent)
        set_tracer(self.original_tracer)

    def create_agent_and_setup_tracer(self):
        self.agent = HostAgent()
        self.span_recorder = StanRecorder(self.agent)
        self.tracer = InstanaTracer(recorder=self.span_recorder)
        set_agent(self.agent)
        set_tracer(self.tracer)

    def test_secrets(self):
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent.options, 'secrets_matcher'))
        self.assertEqual(self.agent.options.secrets_matcher, 'contains-ignore-case')
        self.assertTrue(hasattr(self.agent.options, 'secrets_list'))
        self.assertEqual(self.agent.options.secrets_list, ['key', 'pass', 'secret'])

    def test_options_have_extra_http_headers(self):
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent, 'options'))
        self.assertTrue(hasattr(self.agent.options, 'extra_http_headers'))

    def test_has_options(self):
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent, 'options'))
        self.assertTrue(isinstance(self.agent.options, StandardOptions))

    def test_agent_default_log_level(self):
        self.create_agent_and_setup_tracer()
        self.assertEqual(self.agent.options.log_level, logging.WARNING)

    def test_agent_instana_debug(self):
        os.environ['INSTANA_DEBUG'] = "asdf"
        self.create_agent_and_setup_tracer()
        self.assertEqual(self.agent.options.log_level, logging.DEBUG)

    def test_agent_instana_service_name(self):
        os.environ['INSTANA_SERVICE_NAME'] = "greycake"
        self.create_agent_and_setup_tracer()
        self.assertEqual(self.agent.options.service_name, "greycake")

    @patch.object(requests.Session, "put")
    def test_announce_is_successful(self, mock_requests_session_put):
        test_pid = 4242
        test_process_name = 'test_process'
        test_process_args = ['-v', '-d']
        test_agent_uuid = '83bf1e09-ab16-4203-abf5-34ee0977023a'

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.content = (
                '{'
                f'  "pid": {test_pid}, '
                f'  "agentUuid": "{test_agent_uuid}"'
                '}')

        # This mocks the call to self.agent.client.put
        mock_requests_session_put.return_value = mock_response

        self.create_agent_and_setup_tracer()
        d = Discovery(pid=test_pid,
                      name=test_process_name, args=test_process_args)
        payload = self.agent.announce(d)

        self.assertIn('pid', payload)
        self.assertEqual(test_pid, payload['pid'])

        self.assertIn('agentUuid', payload)
        self.assertEqual(test_agent_uuid, payload['agentUuid'])


    @patch.object(requests.Session, "put")
    def test_announce_fails_with_non_200(self, mock_requests_session_put):
        test_pid = 4242
        test_process_name = 'test_process'
        test_process_args = ['-v', '-d']
        test_agent_uuid = '83bf1e09-ab16-4203-abf5-34ee0977023a'

        mock_response = MagicMock()
        mock_response.status_code = 404
        mock_response.content = ''
        mock_requests_session_put.return_value = mock_response

        self.create_agent_and_setup_tracer()
        d = Discovery(pid=test_pid,
                      name=test_process_name, args=test_process_args)
        with self.assertLogs(logger, level='DEBUG') as log:
            payload = self.agent.announce(d)
        self.assertIsNone(payload)
        self.assertEqual(len(log.output), 1)
        self.assertEqual(len(log.records), 1)
        self.assertIn('response status code', log.output[0])
        self.assertIn('is NOT 200', log.output[0])


    @patch.object(requests.Session, "put")
    def test_announce_fails_with_non_json(self, mock_requests_session_put):
        test_pid = 4242
        test_process_name = 'test_process'
        test_process_args = ['-v', '-d']
        test_agent_uuid = '83bf1e09-ab16-4203-abf5-34ee0977023a'

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.content = ''
        mock_requests_session_put.return_value = mock_response

        self.create_agent_and_setup_tracer()
        d = Discovery(pid=test_pid,
                      name=test_process_name, args=test_process_args)
        with self.assertLogs(logger, level='DEBUG') as log:
            payload = self.agent.announce(d)
        self.assertIsNone(payload)
        self.assertEqual(len(log.output), 1)
        self.assertEqual(len(log.records), 1)
        self.assertIn('response is not JSON', log.output[0])

    @patch.object(requests.Session, "put")
    def test_announce_fails_with_empty_list_json(self, mock_requests_session_put):
        test_pid = 4242
        test_process_name = 'test_process'
        test_process_args = ['-v', '-d']
        test_agent_uuid = '83bf1e09-ab16-4203-abf5-34ee0977023a'

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.content = '[]'
        mock_requests_session_put.return_value = mock_response

        self.create_agent_and_setup_tracer()
        d = Discovery(pid=test_pid,
                      name=test_process_name, args=test_process_args)
        with self.assertLogs(logger, level='DEBUG') as log:
            payload = self.agent.announce(d)
        self.assertIsNone(payload)
        self.assertEqual(len(log.output), 1)
        self.assertEqual(len(log.records), 1)
        self.assertIn('payload has no fields', log.output[0])


    @patch.object(requests.Session, "put")
    def test_announce_fails_with_missing_pid(self, mock_requests_session_put):
        test_pid = 4242
        test_process_name = 'test_process'
        test_process_args = ['-v', '-d']
        test_agent_uuid = '83bf1e09-ab16-4203-abf5-34ee0977023a'

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.content = (
                '{'
                f'  "agentUuid": "{test_agent_uuid}"'
                '}')
        mock_requests_session_put.return_value = mock_response

        self.create_agent_and_setup_tracer()
        d = Discovery(pid=test_pid,
                      name=test_process_name, args=test_process_args)
        with self.assertLogs(logger, level='DEBUG') as log:
            payload = self.agent.announce(d)
        self.assertIsNone(payload)
        self.assertEqual(len(log.output), 1)
        self.assertEqual(len(log.records), 1)
        self.assertIn('response payload has no pid', log.output[0])


    @patch.object(requests.Session, "put")
    def test_announce_fails_with_missing_uuid(self, mock_requests_session_put):
        test_pid = 4242
        test_process_name = 'test_process'
        test_process_args = ['-v', '-d']
        test_agent_uuid = '83bf1e09-ab16-4203-abf5-34ee0977023a'

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.content = (
                '{'
                f'  "pid": {test_pid} '
                '}')
        mock_requests_session_put.return_value = mock_response

        self.create_agent_and_setup_tracer()
        d = Discovery(pid=test_pid,
                      name=test_process_name, args=test_process_args)
        with self.assertLogs(logger, level='DEBUG') as log:
            payload = self.agent.announce(d)
        self.assertIsNone(payload)
        self.assertEqual(len(log.output), 1)
        self.assertEqual(len(log.records), 1)
        self.assertIn('response payload has no agentUuid', log.output[0])


    @patch.object(requests.Session, "get")
    def test_agent_connection_attempt(self, mock_requests_session_get):
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_requests_session_get.return_value = mock_response

        self.create_agent_and_setup_tracer()
        host = self.agent.options.agent_host
        port = self.agent.options.agent_port
        msg = f"Instana host agent found on {host}:{port}"
        
        with self.assertLogs(logger, level='DEBUG') as log:
            result = self.agent.is_agent_listening(host, port)

        self.assertTrue(result)
        self.assertIn(msg, log.output[0])


    @patch.object(requests.Session, "get")
    def test_agent_connection_attempt_fails_with_404(self, mock_requests_session_get):
        mock_response = MagicMock()
        mock_response.status_code = 404
        mock_requests_session_get.return_value = mock_response

        self.create_agent_and_setup_tracer()
        host = self.agent.options.agent_host
        port = self.agent.options.agent_port
        msg = "The attempt to connect to the Instana host agent on " \
              f"{host}:{port} has failed with an unexpected status code. " \
              f"Expected HTTP 200 but received: {mock_response.status_code}"

        with self.assertLogs(logger, level='DEBUG') as log:
            result = self.agent.is_agent_listening(host, port)

        self.assertFalse(result)
        self.assertIn(msg, log.output[0])



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2018

import os
import pytest

testenv = {}

"""
Cassandra Environment
"""
testenv['cassandra_host'] = os.environ.get('CASSANDRA_HOST', '127.0.0.1')
testenv['cassandra_username'] = os.environ.get('CASSANDRA_USERNAME', 'Administrator')
testenv['cassandra_password'] = os.environ.get('CASSANDRA_PASSWORD', 'password')

"""
CouchDB Environment
"""
testenv['couchdb_host'] = os.environ.get('COUCHDB_HOST', '127.0.0.1')
testenv['couchdb_username'] = os.environ.get('COUCHDB_USERNAME', 'Administrator')
testenv['couchdb_password'] = os.environ.get('COUCHDB_PASSWORD', 'password')

"""
MySQL Environment
"""
if 'MYSQL_HOST' in os.environ:
    testenv['mysql_host'] = os.environ['MYSQL_HOST']
else:
    testenv['mysql_host'] = '127.0.0.1'

testenv['mysql_port'] = int(os.environ.get('MYSQL_PORT', '3306'))
testenv['mysql_db'] = os.environ.get('MYSQL_DATABASE', 'instana_test_db')
testenv['mysql_user'] = os.environ.get('MYSQL_USER', 'root')
testenv['mysql_pw'] = os.environ.get('MYSQL_ROOT_PASSWORD', 'passw0rd')

"""
PostgreSQL Environment
"""
testenv['postgresql_host'] = os.environ.get('POSTGRES_HOST', '127.0.0.1')
testenv['postgresql_port'] = int(os.environ.get('POSTGRES_PORT', '5432'))
testenv['postgresql_db'] = os.environ.get('POSTGRES_DB', 'instana_test_db')
testenv['postgresql_user'] = os.environ.get('POSTGRES_USER', 'root')
testenv['postgresql_pw'] = os.environ.get('POSTGRES_PW', 'passw0rd')

"""
Redis Environment
"""
testenv['redis_host'] = os.environ.get('REDIS_HOST', '127.0.0.1')

"""
MongoDB Environment
"""
testenv['mongodb_host'] = os.environ.get('MONGO_HOST', '127.0.0.1')
testenv['mongodb_port'] = os.environ.get('MONGO_PORT', '27017')
testenv['mongodb_user'] = os.environ.get('MONGO_USER', None)
testenv['mongodb_pw'] = os.environ.get('MONGO_PW', None)


def drop_log_spans_from_list(spans):
    """
    Log spans may occur randomly in test runs because of various intentional errors (for testing). This
    helper method will remove all of the log spans from <spans> and return the remaining list.  Helpful
    for those tests where we are not testing log spans - where log spans are just noise.
    @param spans: the list of spans to filter
    @return: a filtered list of spans
    """
    new_list = []
    for span in spans:
        if span.n != 'log':
            new_list.append(span)
    return new_list


def fail_with_message_and_span_dump(msg, spans):
    """
    Helper method to fail a test when the number of spans isn't what was expected.  This helper
    will print <msg> and dump the list of spans in <spans>.

    @param msg: Descriptive message to print with the failure
    @param spans: the list of spans to dump
    @return: None
    """
    span_count = len(spans)
    span_dump = "\nDumping all collected spans (%d) -->\n" % span_count
    if span_count > 0:
        for span in spans:
            span.stack = '<snipped>'
            span_dump += repr(span) + '\n'
    pytest.fail(msg + span_dump, True)


def get_first_span_by_name(spans, name):
    """
    Get the first span in <spans> that has a span.n value of <name>
    @param spans: the list of spans to search
    @param name: the name to search for
    @return: Span or None if nothing found
    """
    for span in spans:
        if span.n == name:
            return span
    return None


def get_first_span_by_filter(spans, filter):
    """
    Get the first span in <spans> that matches <filter>

    Example:
    filter = lambda span: span.n == "tornado-server" and span.data["http"]["status"] == 301
    tornado_301_span = get_first_span_by_filter(spans, filter)

    @param spans: the list of spans to search
    @param filter: the filter to search by
    @return: Span or None if nothing matched
    """
    for span in spans:
        if filter(span) is True:
            return span
    return None


def get_spans_by_filter(spans, filter):
    """
    Get all spans in <spans> that matches <filter>

    Example:
    filter = lambda span: span.n == "tornado-server" and span.data["http"]["status"] == 301
    tornado_301_spans = get_spans_by_filter(spans, filter)

    @param spans: the list of spans to search
    @param filter: the filter to search by
    @return: list of spans
    """
    results = []
    for span in spans:
        if filter(span) is True:
            results.append(span)
    return results


def launch_traced_request(url):
    import requests
    from instana.log import logger
    from instana.singletons import tracer

    logger.warn("Launching request with a root SDK span name of 'launch_traced_request'")

    with tracer.start_active_span('launch_traced_request'):
        response = requests.get(url)

    return response
